/******************************************************************************
 * Class Name:CustomerServiceAgentBean.java
 * Description:Bean Implementation for CustomerServiceAgent.
 * @author Mandar S. Chitnis, Pravin S. Tiwari, Lakshmi AM.
 * @version 11.5
 * Copyright (c) by Sams Publishing. All Rights Reserved.
******************************************************************************/
package com.sams.learnweblogic7.airlines.sessionbean;

import javax.ejb.*;
import java.util.*;
import javax.naming.*;
import javax.rmi.*;
import com.sams.learnweblogic7.airlines.businessobject.*;
import com.sams.learnweblogic7.airlines.constants.*;
import com.sams.learnweblogic7.airlines.entitybean.*;
import com.sams.learnweblogic7.airlines.exception.*;

public class CustomerServiceAgentBean implements javax.ejb.SessionBean
{
  public CustomerServiceAgentBean()
  {
  }

  // ---------------------------------------------------------------
  // private fields
  private SessionContext ctx;
  private String sep=System.getProperty( "line.separator" );

  // ---------------------------------------------------------------
  // SessionBean interface implementation
  public void ejbActivate()
  {
  }

  public void ejbPassivate()
  {
  }

  public void ejbRemove()
  {
  }

  public void setSessionContext(SessionContext ctx)
  {
    this.ctx=ctx;
  }

  // ---------------------------------------------------------------
  // create methods
  public void ejbCreate()
  {
  }

  // ---------------------------------------------------------------
  // business methods
  public MVCAppValueObject authenticateUser(MVCAppValueObject objValue)
  throws GenericException
  {
    String errorDesc="";
    PassengerHome passengerHomeObj=null;
    PassengerInterface passengerRemoteObj=null;
    MVCAppValueObject returnValueObject=null;

    try
    {
      passengerHomeObj=lookupPassengerHome();

      String profileID=(String)objValue.get(JspFieldConstants.LOGIN_PAGE_USER_NAME);
      String password=(String)objValue.get(JspFieldConstants.LOGIN_PAGE_PASSWORD);

      passengerRemoteObj=passengerHomeObj.findByPassword(profileID, password);

    }catch(javax.ejb.ObjectNotFoundException e)
    {
      errorDesc="Error while Authenticating User (Please check if User ID and Password are valid)" + sep;
      throw new GenericException(errorDesc);
    }
    catch(Exception exp)
    {
      throw new GenericException(exp);
    }

    if (passengerRemoteObj==null)
    {
      errorDesc="Error while Authenticating User (Please check if User ID and Password are valid)" + sep;
      throw new GenericException(errorDesc);
    }

    try
    {
      returnValueObject=createPassengerMVCValueObject(passengerRemoteObj);

        }catch(Exception exp)
    {
      errorDesc="Error while Authenticating User .... Internal error occurred"+ sep;
      errorDesc=errorDesc + " Please try later ";
      throw new GenericException(errorDesc,exp);
    }

    return returnValueObject;
  }

  public MVCAppValueObject createUserProfile(MVCAppValueObject objValue)
  throws GenericException
  {
    String errorDesc="";
    MVCAppValueObject valueObj=new MVCAppValueObject();
    PassengerHome passengerHomeObj=null;
    PassengerInterface passengerRemoteObj=null;
    String profileID="";
    String password="";
    String firstName="";
    String lastName="";
    String address="";
    String phoneNo="";
    String emailID="";
    String cardNo="";
    String cardType="";
    String ccMonth="";
    String ccYear="";
    try
    {
      passengerHomeObj=lookupPassengerHome();

        }catch(Exception exp)
    {
      errorDesc="Error while creating the User Profile .... Internal error occurred" + sep;
      errorDesc=errorDesc + " Please try later ";
      exp.printStackTrace();
      throw new GenericException(errorDesc,exp);
    }

    profileID=(String)objValue.get(JspFieldConstants.REGISTER_USER_PROFILE_PAGE_USER_ID);
    password=(String)objValue.get(JspFieldConstants.REGISTER_USER_PROFILE_PAGE_PASSWORD);
    firstName=(String)objValue.get(JspFieldConstants.REGISTER_USER_PROFILE_PAGE_FIRST_NAME);
    lastName=(String)objValue.get(JspFieldConstants.REGISTER_USER_PROFILE_PAGE_LAST_NAME);
    address=(String)objValue.get(JspFieldConstants.REGISTER_USER_PROFILE_PAGE_ADDRESS);
    phoneNo=(String)objValue.get(JspFieldConstants.REGISTER_USER_PROFILE_PAGE_PHONE_NO);
    emailID=(String)objValue.get(JspFieldConstants.REGISTER_USER_PROFILE_PAGE_EMAIL_ADDRESS);
    cardNo=(String)objValue.get(JspFieldConstants.REGISTER_USER_PROFILE_PAGE_CC_NO);
    cardType=(String)objValue.get(JspFieldConstants.REGISTER_USER_PROFILE_PAGE_CC_TYPE);
    ccMonth=(String)objValue.get(JspFieldConstants.REGISTER_USER_PROFILE_PAGE_EXP_MONTH);
    ccYear=(String)objValue.get(JspFieldConstants.REGISTER_USER_PROFILE_PAGE_EXP_YEAR);
    try
    {
      passengerRemoteObj=passengerHomeObj.findByPrimaryKey(profileID);

    }catch(javax.ejb.ObjectNotFoundException e)
    {
      // here do nothing since this exception only means that the profleID
      // does not exist and user can go ahead with the registration.
    }
    catch(Exception e)
    {
      errorDesc="Error while creating the User Profile ....Internal error occurred" + sep;
      errorDesc=errorDesc + " Please try later ";
      throw new GenericException(errorDesc,e);
    }

    if (passengerRemoteObj !=null)
    {
      errorDesc="Error while creating the User Profile ....User Name already exists " + sep;
      errorDesc=errorDesc + " Please enter a different User Name ";
      throw new GenericException(errorDesc);
    }

    try
    {
     passengerRemoteObj=(PassengerInterface)PortableRemoteObject.narrow(passengerHomeObj.create(profileID,password,firstName,lastName,
      address,phoneNo,emailID, cardNo,cardType,Integer.parseInt(ccMonth),
      Integer.parseInt(ccYear)),PassengerInterface.class);

      PassengerProfile profileObj=new PassengerProfile();

      profileObj.setProfileID(profileID);
      profileObj.setPassword(password);
      profileObj.setFirstName(firstName);
      profileObj.setLastName(lastName);
      profileObj.setAddress(address);
      profileObj.setTelNo(phoneNo);
      profileObj.setEmailID(emailID);
      profileObj.setCardNumber(cardNo);
      profileObj.setCardType(cardType);
      profileObj.setExpirationMonth(Integer.parseInt(ccMonth));
      profileObj.setExpirationYear(Integer.parseInt(ccYear));

      valueObj.put(MVCConstants.PASSENGER_PROFILE, profileObj);

        }catch(Exception exp)
    {
      errorDesc="Error while creating the User Profile ....Internal error occurred" + sep;
      errorDesc=errorDesc + " Please try later ";
      throw new GenericException(errorDesc,exp);
    }
    return valueObj;
  }

  public MVCAppValueObject viewAvailableFlights(MVCAppValueObject objValue)
  throws GenericException
  {
    //first let's find out if there is a entry for a flight for the selected
   //date getting the FLIGHT home interface.
    String errorDesc="";
    MVCAppValueObject valueObj=new MVCAppValueObject();
    Collection col=null;

    try
    {
      FlightHome flightHomeObj=lookupFlightHome();
      String fromLocation=(String)objValue.get(JspFieldConstants.CHECK_AVAIL_FLIGHTS_PAGE_DEP_CITY);
	  String toLocation=(String)objValue.get(JspFieldConstants.CHECK_AVAIL_FLIGHTS_PAGE_ARR_CITY);
      String day=(String)objValue.get(JspFieldConstants.CHECK_AVAIL_FLIGHTS_PAGE_DEP_DAY);
      String month=(String)objValue.get(JspFieldConstants.CHECK_AVAIL_FLIGHTS_PAGE_DEP_MONTH);
      String year=(String)objValue.get(JspFieldConstants.CHECK_AVAIL_FLIGHTS_PAGE_DEP_YEAR);
      Calendar buildSqlDate=Calendar.getInstance();
	  int parseYear = Integer.parseInt(year);
	  int parseMonth = Integer.parseInt(month);
	  int parseDay = Integer.parseInt(day);
	  //since the util date starts month from 0, deducting 1 from parsemonth
      buildSqlDate.set(parseYear,parseMonth-1,parseDay);
	  java.util.Date utilDate = buildSqlDate.getTime();
	  java.sql.Date bookDate=new java.sql.Date(utilDate.getTime());
      col=flightHomeObj.findAvailableFlights(fromLocation, toLocation, bookDate);
    }catch(Exception exp)
    {
	  errorDesc="Error while searching the available flights ...*.Internal error occurred" + sep;
	  errorDesc=errorDesc + " Please try later ";
	  throw new GenericException(errorDesc,exp);
    }
    if(col.isEmpty())
    {
      errorDesc="No Flights available for the selected date or location!";
      throw new GenericException(errorDesc);
    }
    try
    {
      Iterator it=col.iterator();
	  System.out.println("Collection obtained - col.iterator"+it);
      Vector flightVec=new Vector();
	  int temp_cnt=0;
      while (it.hasNext())

      {
		temp_cnt++;
        FlightInterface flightRemoteObj=(FlightInterface)PortableRemoteObject.narrow(it.next(), FlightInterface.class);
        flightVec.addElement(createFlightMVCValueObject(flightRemoteObj));
      }
      valueObj.put(MVCConstants.ALL_FLIGHTS_INFORMATION, flightVec);
    }catch(Exception exp)
    {
      errorDesc="Error while searching the available flights ....Internal error occurred" + sep;
      errorDesc=errorDesc + " Please try later ";
      throw new GenericException(errorDesc,exp);
    }
      return valueObj;
  }

  private MVCAppValueObject createPassengerMVCValueObject (PassengerInterface passengerRemoteObj) throws Exception
  {
        PassengerProfile profileObj=new PassengerProfile();

        profileObj.setProfileID(passengerRemoteObj.getProfileId());
        profileObj.setPassword(passengerRemoteObj.getPassword());
        profileObj.setFirstName(passengerRemoteObj.getFirstName());
        profileObj.setLastName(passengerRemoteObj.getLastName());
        profileObj.setAddress(passengerRemoteObj.getAddress());
        profileObj.setTelNo(passengerRemoteObj.getTelNo());
        profileObj.setCardNumber(passengerRemoteObj.getCardNumber());
        profileObj.setCardType(passengerRemoteObj.getCardType());
        profileObj.setExpirationMonth(passengerRemoteObj.getExpirationMonth());
        profileObj.setExpirationYear(passengerRemoteObj.getExpirationYear());

        MVCAppValueObject valueObj=new MVCAppValueObject();
        valueObj.put(MVCConstants.PASSENGER_PROFILE, profileObj);

        return valueObj;
    }

  private FlightDetails createFlightMVCValueObject (FlightInterface flightRemoteObj) throws Exception
  {
      FlightDetails flightDetailsBO=new FlightDetails();

      flightDetailsBO.setAirlineID(flightRemoteObj.getAirlineId());
      flightDetailsBO.setAirlineName(flightRemoteObj.getAirlineName());
      flightDetailsBO.setFlightDepartureTime(flightRemoteObj.getDepartureTime());
      flightDetailsBO.setFlightArrivalTime(flightRemoteObj.getArrivalTime());
      flightDetailsBO.setFlightID(flightRemoteObj.getFlightId());
      flightDetailsBO.setFromLocation(flightRemoteObj.getFromLocation());
      flightDetailsBO.setToLocation(flightRemoteObj.getToLocation());
      flightDetailsBO.setPrice(flightRemoteObj.getPrice());
      flightDetailsBO.setDepartureDate(flightRemoteObj.getFlightDepartureDate());
      return flightDetailsBO;
  }

  /**
     * Look up the Airline bean's home interface using JNDI.
     */
    private FlightHome lookupFlightHome()throws NamingException
    {
      Hashtable h=new Hashtable();
      h.put(Context.INITIAL_CONTEXT_FACTORY, MVCConstants.WEBLOGIC_INITIAL_CONTEXT_FACTORY);
      h.put(Context.PROVIDER_URL, MVCConstants.WEBLOGIC_PROVIDER_URL);
      Context ctx=new InitialContext(h);
      try
      {
          Object home=(FlightHome)ctx.lookup("Flight_Home");
          return (FlightHome) PortableRemoteObject.narrow(home, FlightHome.class);
      }
      catch (NamingException ne)
      {
          System.out.println("Unable to lookup the EJBHome.  Verify the ejb JNDI name FlightHome on the WebLogic server");
          ne.printStackTrace();
          throw ne;
      }
    }

  /**
     * Look up the Airline bean's home interface using JNDI.
     */
    private PassengerHome lookupPassengerHome()throws NamingException
    {
        Hashtable h=new Hashtable();
        h.put(Context.INITIAL_CONTEXT_FACTORY, MVCConstants.WEBLOGIC_INITIAL_CONTEXT_FACTORY);
        h.put(Context.PROVIDER_URL, MVCConstants.WEBLOGIC_PROVIDER_URL);
        Context ctx=new InitialContext(h);
        System.out.println("Initial context to look up CMP PassengerHome created "+ctx);
        try
        {
            Object home=(PassengerHome)ctx.lookup("Passenger");
            return (PassengerHome) PortableRemoteObject.narrow(home,PassengerHome.class);
        }
        catch (NamingException ne)
        {
            System.out.println("Unable to lookup the EJBHome.Verify the ejb JNDI name CustomerServiceAgentHome on the WebLogic server");
            throw ne;
        }
    }
}
