/******************************************************************************
 * Class Name: TicketSalesAgentBean.java
 * Description:Bean Implementation for TicketSalesAgentBeam.
 * @author Mandar S. Chitnis, Pravin S. Tiwari, Lakshmi AM.
 * @version 11.5
 * Copyright (c) by Sams Publishing. All Rights Reserved.
******************************************************************************/
package com.sams.learnweblogic7.airlines.sessionbean;

import java.sql.Date;
import javax.ejb.*;
import java.util.*;
import javax.naming.*;
import javax.rmi.*;
import com.sams.learnweblogic7.airlines.constants.*;
import com.sams.learnweblogic7.airlines.businessobject.*;
import com.sams.learnweblogic7.airlines.util.*;
import com.sams.learnweblogic7.airlines.entitybean.*;
import com.sams.learnweblogic7.airlines.exception.*;

public class TicketSalesAgentBean implements javax.ejb.SessionBean,javax.ejb.SessionSynchronization
{
  // ---------------------------------------------------------------
  // conversational state fields
  public MVCAppValueObject sessionData;
  // ---------------------------------------------------------------
  // private fields
  private SessionContext ctx;
  private String sep=System.getProperty( "line.separator" );

  public TicketSalesAgentBean()
  {
  }

  // ---------------------------------------------------------------
  // SessionBean interface implementation
  public void ejbActivate()
  {
  }

  public void ejbPassivate()
  {
  }

  public void ejbRemove()
  {
  }

  public void setSessionContext(SessionContext ctx)
  {
    this.ctx=ctx;
  }

  // ---------------------------------------------------------------
  // SessionSynchronization interface implementation
  public void afterBegin()
  {
  }

  public void afterCompletion(boolean committed)
  {
  }

  public void beforeCompletion()
  {
  }

  // ---------------------------------------------------------------
  // create methods
  public void ejbCreate()
  {
    sessionData=new MVCAppValueObject();
  }

  // ---------------------------------------------------------------
  // business methods
  public MVCAppValueObject confirmBooking() throws GenericException
  {
    String errorDesc="";
    MVCAppValueObject returnValueObj=new MVCAppValueObject();
	
    // get the data from the session
    PassengerProfile passengerProfileBO=(PassengerProfile)sessionData.get(MVCConstants.PASSENGER_PROFILE);
    FlightDetails flightDetailsBO=(FlightDetails)sessionData.get(MVCConstants.PASSENGER_FLIGHT_DETAILS);

    if (passengerProfileBO==null || flightDetailsBO==null)
    {
      errorDesc="Error while Confirming User Booking....Internal error occurred" + sep;
      errorDesc=errorDesc + " Please try later ";
      throw new GenericException(errorDesc);
    }

    try
    {
      // first, charge the credit card
      String ccNo=passengerProfileBO.getCardNumber();
      String ccType=passengerProfileBO.getCardType();
      String fullName=passengerProfileBO.getFirstName() + passengerProfileBO.getLastName();
      int expiryMonth=passengerProfileBO.getExpirationMonth();
      int expiryYear=passengerProfileBO.getExpirationYear();
      boolean ccFlag=new CreditCardProcessing().process(ccNo, ccType, fullName,expiryMonth, expiryYear);

    }
    catch(Exception exp)
    {
      errorDesc="Error while Processing the Credit Card" + sep;
      errorDesc=errorDesc + " Please enter the correct Credit Card Details ";
      throw new GenericException(errorDesc,exp);
    }

    try
    {
      // second, insert a row in the TICKETINFO table
      // and return a unique ticket number
      TicketInfoHome ticketInfoHomeObj=lookupTicketInfoHome();
      long ticketID=System.currentTimeMillis ();
      String profileID=passengerProfileBO.getProfileID();
      String status="confirmed";
      int flightId=flightDetailsBO.getFlightID();
      Date departureDate=flightDetailsBO.getDepartureDate();
      TicketInfoInterface ticketInfoRemoteObj=
       (TicketInfoInterface)PortableRemoteObject.narrow(ticketInfoHomeObj.create(ticketID,profileID,status,flightId,
        departureDate),TicketInfoInterface.class);
      TicketDetails ticketDetailsBO=new TicketDetails();
      ticketDetailsBO.setTicketID(ticketID);
      ticketDetailsBO.setProfileID(profileID);
      ticketDetailsBO.setStatus(status);
      ticketDetailsBO.setFlightID(flightId);
      ticketDetailsBO.setDepartureDate(departureDate);

      // now put the ticket details in the session
      sessionData.put(MVCConstants.PASSENGER_TICKET_DETAILS, ticketDetailsBO);
      // also put in in the return object
      returnValueObj.put(MVCConstants.PASSENGER_TICKET_DETAILS, ticketDetailsBO);
      // commit the transaction
      // send the email with ticket details
      String emailAddress=passengerProfileBO.getEmailID();
      boolean emailFlag=new Email().sendEmail(emailAddress, "Ticket Confirmtion", ticketID+"");
    }
    catch(Exception exp)
    {
      errorDesc="Error while Confirming User Booking.... Internal error occurred" + sep;
      errorDesc=errorDesc + " Please try later ";
      throw new GenericException(errorDesc,exp);
    }
    return returnValueObj;
  }

  public MVCAppValueObject cancelBooking() throws GenericException
  {
    String errorDesc="";
    MVCAppValueObject returnValueObj=new MVCAppValueObject();
    try
    {
      // remove the ticket ordered from the session
      sessionData.remove(MVCConstants.PASSENGER_TICKET_DETAILS);
    }
    catch(Exception exp)
    {
      errorDesc="Error while Canceling User Booking.... Internal error occurred" + sep;
      errorDesc=errorDesc + " Please try later ";
      throw new GenericException(errorDesc,exp);
    }
    return returnValueObj;
  }

  public MVCAppValueObject addOrder(MVCAppValueObject objValue) throws GenericException
  {
    MVCAppValueObject returnValueObj=new MVCAppValueObject();
    try
    {
      // first get the flight id for a date that the user is interested in buying
      String strFlightID=(String)objValue.get(JspFieldConstants.DISPLAY_AVAIL_FLIGHTS_PAGE_FLIGHT_ID);
      int flightID=Integer.parseInt(strFlightID);
      String strDepartureDate=(String)objValue.get(JspFieldConstants.DISPLAY_AVAIL_FLIGHTS_PAGE_DEP_DATE);
      java.sql.Date departureDate=java.sql.Date.valueOf(strDepartureDate);

      // add the selected flight details in the user session
      FlightHome flightHomeObj=lookupFlightHome();
      FlightInterface flightRemoteObj=flightHomeObj.findFlightForADate(flightID, departureDate);
      FlightDetails flightDetailsBO=createFlightMVCValueObject(flightRemoteObj);
      sessionData.put(MVCConstants.PASSENGER_FLIGHT_DETAILS, flightDetailsBO);
    }
    catch(Exception exp)
    {
      exp.printStackTrace();
      throw new GenericException(exp);
    }
    return returnValueObj;
  }

  public void putUserInformation(MVCAppValueObject objValue)
  {
    PassengerProfile passengerProfileObj=(PassengerProfile)objValue.get(MVCConstants.PASSENGER_PROFILE);
    sessionData.put(MVCConstants.PASSENGER_PROFILE, passengerProfileObj);
  }

  public MVCAppValueObject getUserInformation() throws GenericException
  {
    String errorDesc="";
    MVCAppValueObject userInfoValueObject=new MVCAppValueObject();
    PassengerProfile passengerProfileObj=(PassengerProfile)sessionData.get(MVCConstants.PASSENGER_PROFILE);
    if (passengerProfileObj== null)
    {
      errorDesc=" Error while reteiving the User Information.... Internal error occurred " + sep;
      errorDesc=errorDesc + " Please try later ";
      throw new GenericException(errorDesc);
    }
    userInfoValueObject.put(MVCConstants.PASSENGER_PROFILE, passengerProfileObj);

    return userInfoValueObject;
  }

  public MVCAppValueObject getUserTicketInfo()
  {
    Collection col=null;
    FlightHome flightHomeObj=null;
    MVCAppValueObject userInfoValueObject=new MVCAppValueObject();
    PassengerProfile passengerProfileObj=(PassengerProfile)sessionData.get(MVCConstants.PASSENGER_PROFILE);
    Vector tickets=new Vector();

    try
    {
      flightHomeObj=lookupFlightHome();
      TicketInfoHome ticketInfoHomeObj=lookupTicketInfoHome();
      col=ticketInfoHomeObj.findByProfileID(passengerProfileObj.getProfileID());
      if(col.isEmpty())
      {
        System.out.println("No ticket information avaialble ");
      }

      Iterator it=col.iterator();

      while (it.hasNext())
      {

        MVCAppValueObject flightInfoValueObject=new MVCAppValueObject();

        TicketInfoInterface ticketInfoRemoteObj=(TicketInfoInterface) PortableRemoteObject.narrow(it.next(),TicketInfoInterface.class);
        TicketDetails ticketDetailsBO=new TicketDetails();
        ticketDetailsBO.setTicketID(ticketInfoRemoteObj.getTicketId());
        ticketDetailsBO.setProfileID(ticketInfoRemoteObj.getProfileId());
        ticketDetailsBO.setStatus(ticketInfoRemoteObj.getStatus());
        ticketDetailsBO.setFlightID(ticketInfoRemoteObj.getFlightId());
        ticketDetailsBO.setDepartureDate(ticketInfoRemoteObj.getDepartureDate());
        flightInfoValueObject.put(MVCConstants.PASSENGER_TICKET_DETAILS,ticketDetailsBO);

        // now that you have the ticket information, get the flight details
        //for that ticket
        FlightInterface flightRemoteObj=flightHomeObj.findFlightForADate(ticketInfoRemoteObj.getFlightId(),ticketInfoRemoteObj.getDepartureDate());
        FlightDetails flightDetailsBO=createFlightMVCValueObject(flightRemoteObj);
        flightInfoValueObject.put(MVCConstants.PASSENGER_FLIGHT_DETAILS,flightDetailsBO);
        tickets.addElement(flightInfoValueObject);
      }
    }
    catch(Exception e)
    {
      System.out.println("Error in TicketSalesAgent - "+e);
      e.printStackTrace();
    }

    userInfoValueObject.put(MVCConstants.PASSENGER_TICKET_INFO, tickets);
    return userInfoValueObject;
  }
  // ---------------------------------------------------------------
  // get/set conversational state
  public MVCAppValueObject getSessionData()
  {
    return this.sessionData;
  }

  public void setSessionData(MVCAppValueObject sessionData)
  {
    this.sessionData=sessionData;
  }

  // --------------------------------------------------------
  /**
  * Look up the Airline bean's home interface using JNDI
  */
  private TicketInfoHome lookupTicketInfoHome()	throws NamingException
  {
    Hashtable h=new Hashtable();
    h.put(Context.INITIAL_CONTEXT_FACTORY,MVCConstants.WEBLOGIC_INITIAL_CONTEXT_FACTORY);
    h.put(Context.PROVIDER_URL, MVCConstants.WEBLOGIC_PROVIDER_URL);
    Context ctx=new InitialContext(h);
    try
    {
      Object home=(TicketInfoHome)ctx.lookup("TicketInfo");
      return (TicketInfoHome) PortableRemoteObject.narrow(home,TicketInfoHome.class);

    }
    catch (NamingException ne)
    {
      System.out.println("Unable to lookup the EJBHome.  Verify the ejb JNDI name CustomerServiceAgentHome on the WebLogic server");
      throw ne;
    }
  }

  /**
  * Look up the Flight bean's home interface using JNDI.
  */
  private FlightHome lookupFlightHome()	throws NamingException
  {
    Hashtable h=new Hashtable();
    h.put(Context.INITIAL_CONTEXT_FACTORY,MVCConstants.WEBLOGIC_INITIAL_CONTEXT_FACTORY);
    h.put(Context.PROVIDER_URL, MVCConstants.WEBLOGIC_PROVIDER_URL);
    Context ctx=new InitialContext(h);
    try
    {
      Object home=(FlightHome)ctx.lookup("Flight_Home");
      return (FlightHome) PortableRemoteObject.narrow(home, FlightHome.class);
    }
    catch (NamingException ne)
    {
      System.out.println("Unable to lookup the EJBHome. Verify the ejb JNDI name FlightHome on the WebLogic server");
      throw ne;
    }
  }

  private FlightDetails createFlightMVCValueObject (FlightInterface flightRemoteObj) throws Exception
  {
    FlightDetails flightDetailsBO=new FlightDetails();

    flightDetailsBO.setAirlineID(flightRemoteObj.getAirlineId());
    flightDetailsBO.setAirlineName(flightRemoteObj.getAirlineName());
    flightDetailsBO.setFlightDepartureTime(flightRemoteObj.getDepartureTime());
    flightDetailsBO.setFlightID(flightRemoteObj.getFlightId());
    flightDetailsBO.setFromLocation(flightRemoteObj.getFromLocation());
    flightDetailsBO.setToLocation(flightRemoteObj.getToLocation());
    flightDetailsBO.setPrice(flightRemoteObj.getPrice());
    flightDetailsBO.setFlightDuration(flightRemoteObj.getDuration());
    flightDetailsBO.setDepartureDate(flightRemoteObj.getFlightDepartureDate());
    return flightDetailsBO;
  }
}
